import { Inject, Injectable } from '@nestjs/common';
import { OnEvent } from '@nestjs/event-emitter';
import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { USER_MESSAGE_PUBLISHER } from '@modules/user/user.di-tokens';
import { UserCreatedDomainEvent } from '../../domain/events/user-created.domain-event';
import { UserCreatedIntegrationEvent } from '../events/user-created.integration-event';
import { USER_CREATED_ROUTING_KEY } from '@modules/user/user.constants';

@Injectable()
export class PublishMessageWhenUserIsCreatedDomainEventHandler {
  constructor(
    @Inject(USER_MESSAGE_PUBLISHER)
    private readonly messagePublisher: MessagePublisherPort,
  ) {}

  @OnEvent(UserCreatedDomainEvent.name, { async: true, promisify: true })
  async handle(event: UserCreatedDomainEvent): Promise<any> {
    const userCreatedIntegrationEvent = new UserCreatedIntegrationEvent({
      id: event.aggregateId,
      firstName: event.firstName,
      lastName: event.lastName,
      email: event.email,
      phone: event.phone,
      metadata: event.metadata,
    });
    this.messagePublisher.publish(
      USER_CREATED_ROUTING_KEY,
      JSON.stringify(userCreatedIntegrationEvent),
    );
  }
}
