import { Test, TestingModule } from '@nestjs/testing';
import { USER_MESSAGE_PUBLISHER } from '@modules/user/user.di-tokens';
import { PublishMessageWhenUserIsUpdatedDomainEventHandler } from '@modules/user/core/application/event-handlers/publish-message-when-user-is-updated.domain-event-handler';
import { UserUpdatedDomainEvent } from '@modules/user/core/domain/events/user-updated.domain-event';

const mockMessagePublisher = {
  publish: jest.fn().mockImplementation(),
};

describe('Publish message when user is updated domain event handler', () => {
  let publishMessageWhenUserIsUpdatedDomainEventHandler: PublishMessageWhenUserIsUpdatedDomainEventHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: USER_MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
        PublishMessageWhenUserIsUpdatedDomainEventHandler,
      ],
    }).compile();

    publishMessageWhenUserIsUpdatedDomainEventHandler =
      module.get<PublishMessageWhenUserIsUpdatedDomainEventHandler>(
        PublishMessageWhenUserIsUpdatedDomainEventHandler,
      );
  });

  it('should publish a message', () => {
    jest.spyOn(mockMessagePublisher, 'publish');
    const userUpdatedDomainEvent: UserUpdatedDomainEvent = {
      id: 'some-domain-event-id',
      aggregateId: 'some-aggregate-id',
      firstName: 'Jane',
      lastName: 'Doe',
      email: 'jane.doe@email.com',
      phone: '+33611223344',
      metadata: {
        timestamp: new Date('2023-06-28T05:00:00Z').getTime(),
        correlationId: 'some-correlation-id',
      },
    };
    publishMessageWhenUserIsUpdatedDomainEventHandler.handle(
      userUpdatedDomainEvent,
    );
    expect(publishMessageWhenUserIsUpdatedDomainEventHandler).toBeDefined();
    expect(mockMessagePublisher.publish).toHaveBeenCalledTimes(1);
    expect(mockMessagePublisher.publish).toHaveBeenCalledWith(
      'user.updated',
      '{"id":"some-aggregate-id","metadata":{"correlationId":"some-correlation-id","timestamp":1687928400000},"firstName":"Jane","lastName":"Doe","email":"jane.doe@email.com","phone":"+33611223344"}',
    );
  });
});
